-------------------------------------------------------------------------------
--
--  Working Data
--
-------------------------------------------------------------------------------
local data = {
  player = "",
  buttons = {},
  inCombat = false,
  appliedTo = nil,
  timer = nil,
  enabled = false,
  statusDetail = ""
}

-------------------------------------------------------------------------------
--
--  Constants
--
-------------------------------------------------------------------------------
local ARCANE_SPEC_ID = 62
local NETHER_TEMPEST_ID = 114923
local ACTION_BARS = { "Action", "MultiBarBottomLeft", "MultiBarBottomRight", "MultiBarRight", "MultiBarLeft" }

-------------------------------------------------------------------------------
--
--  Frames
--
-------------------------------------------------------------------------------

local f = CreateFrame("Frame")

-------------------------------------------------------------------------------
--
--  Event and Helper Methods
--
-------------------------------------------------------------------------------

function f:CombatStart()
  data.inCombat = true
end

function f:CombatEnd()
  if (data.timer) then
    data.timer:Cancel()
  end
  data.inCombat = false
  data.appliedTo = nil  
  data.timer = nil
end

function f:IsApplied()
  return data.appliedTo ~= nil
end

function f:HasNetherTempest(spec)
  local hasNetherTempest = false
  
  for talentTier = 1, GetMaxTalentTier() do
    for talentColumn = 1, 3 do
	  local _,name,_,selected,_,spellId = GetTalentInfo(talentTier, talentColumn, spec)

	  if (selected and spellId == NETHER_TEMPEST_ID) then
	    hasNetherTempest = true
		break
	  end
	end

	if (hasNetherTempest) then
      break
	end
  end
  
  return hasNetherTempest
end

function f:DiscoverNetherTempestButtons()
    data.buttons = {}

	for _, barName in pairs(ACTION_BARS) do
		for i = 1, 12 do
			local button = _G[barName .. "Button" .. i]
			local slot = ActionButton_GetPagedID(button) or ActionButton_CalculateAction(button) or button:GetAttribute("action") or 0
			
			if (HasAction(slot)) then
				local actionType, id, _, actionName = GetActionInfo(slot)
				
				if (actionType == "spell" and id == NETHER_TEMPEST_ID) then
				    table.insert(data.buttons, button)
				end
			end
		end
	end
end

function f:DetectSpecialization()
  local specIndex = GetSpecialization()
  local spec = GetSpecializationInfo(specIndex)
  local disabled = false
  
  data.statusDetail = ""

  if (spec ~= ARCANE_SPEC_ID) then
    data.statusDetail = " - Not Arcane"
	disabled = true
  else 
    if (self:HasNetherTempest(specIndex) == false) then
      data.statusDetail = " - Nether Tempest not talented"
	  disabled = true
	end
  end

  if (disabled) then
    data.enabled = false
    data.buttons = {}
    self:UnregisterEvent("PLAYER_REGEN_DISABLED")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
    self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:UnregisterEvent("ACTIONBAR_SLOT_CHANGED")
  else
    data.enabled = true
    self:DiscoverNetherTempestButtons()
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("ACTIONBAR_SLOT_CHANGED")
  end
end

function f:HandleSpellCombatEvent(timestamp, event, hideCaster, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags,
                                  spellId, spellName, spellSchool, auraType, amount)

  if (sourceName == data.player) then
  
    if (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESH") then
	  -- If the player applied nether tempest, then prepare to glow close to it's natural expiration
	  if (spellId == NETHER_TEMPEST_ID) then
        data.appliedTo = destGUID
	    self:DimNetherTempest()

		-- Since it was just applied, we assume the player has the unit it was applied to targeted
        local name, rank, icon, count, dispelType, duration, expires = UnitDebuff("target", spellName, "", "PLAYER")
	
        -- If we had a previous timer, cancel it, as we just refreshed the debuff
		if (data.timer) then
		  data.timer:Cancel()
		end

		-- Schedule a glow to occur close to when nether tempest will expire
		if (duration) then 
		  data.timer = C_Timer.NewTimer(duration - 3, function()
		    self:GlowNetherTempest()
		    data.timer = nil
		  end)
		end
	  end
      
	elseif (event == "SPELL_AURA_REMOVED") then
	  -- Only glow on expiration/removal, if it is off the last target we applied it to
	  -- Otherwise if the player moves the debuff to a new target we would glow the button in error due to ordering of events
	  if (spellId == NETHER_TEMPEST_ID and data.appliedTo == destGUID) then
        data.applied = nil
	    self:GlowNetherTempest()
	  end
    end
  end
end

function f:GlowNetherTempest()
    if (data and data.inCombat) then
      for i, button in ipairs(data.buttons) do
        ActionButton_ShowOverlayGlow(button)
      end
    end
end

function f:DimNetherTempest()
  if (data and data.buttons) then
    for i, button in ipairs(data.buttons) do
      ActionButton_HideOverlayGlow(button)
    end
  end
end

-------------------------------------------------------------------------------
--
--  Initialization
--
-------------------------------------------------------------------------------

f:SetScript("OnEvent", function(self,event,...)

  if (event == "PLAYER_LOGIN") then
    local _,playerClass = UnitClass("player")
	
	-- If the player is not a mage, do no further work
	if (playerClass == "MAGE") then
      data.player = UnitName("player")
      self:DetectSpecialization()
	  self:RegisterUnitEvent("PLAYER_SPECIALIZATION_CHANGED", "player")
	else
	  data.enabled = false
	  data.statusDetail = " - Not a mage"
	end

  elseif (event == "PLAYER_SPECIALIZATION_CHANGED") then
    self:DimNetherTempest()
    self:DetectSpecialization()
  
  elseif (event == "ACTIONBAR_SLOT_CHANGED") then
    self:DiscoverNetherTempestButtons()

  elseif (event == "PLAYER_REGEN_DISABLED") then
    self:CombatStart()
	
	-- Yield the glowing right away while entering combat to
	-- prevent a flash from happening if the player starts combat by applying
	-- nether tempest
	C_Timer.After(0, function()
	  if (self:IsApplied() == false) then
	    self:GlowNetherTempest()
	  end
	end)
	
  elseif (event == "PLAYER_REGEN_ENABLED") then
    self:CombatEnd()
    self:DimNetherTempest()

  elseif (event == "COMBAT_LOG_EVENT_UNFILTERED") then
    self:HandleSpellCombatEvent(...)

  end
end)

-- Add a slash handler for getting the status
SLASH_TEMPESTLIGHT1 = '/tempest'
SlashCmdList["TEMPESTLIGHT"] = function(msg, editBox)
  local status
  if (data.enabled) then status = "Active" else status = "Disabled" end
  
  print("TempestLights: [", status, "]", data.statusDetail)
end

-- Register the initial event to kick everything off
f:RegisterEvent("PLAYER_LOGIN")
